@@ -16,7 +16,6 @@ class TenyksService(object):
 
     irc_message_filters = {}
     name = None
-    direct_only = False
     version = '0.0'
     required_data_fields = ["command", "payload"]
 
@@ -26,18 +25,8 @@ class TenyksService(object):
         self.name = name.lower().replace(' ', '')
         self.logger = logging.getLogger(self.name)
         if self.irc_message_filters:
-            self.re_irc_message_filters = {}
-            for name, regexes in self.irc_message_filters.iteritems():
-                if name not in self.re_irc_message_filters:
-                    self.re_irc_message_filters[name] = []
-                if isinstance(regexes, basestring):
-                    regexes = [regexes]
-                for regex in regexes:
-                    if isinstance(regex, str) or isinstance(regex, unicode):
-                        self.re_irc_message_filters[name].append(
-                            re.compile(regex).match)
-                    else:
-                        self.re_irc_message_filters[name].append(regex)
+            map(lambda f: f._compile_filters(),
+                self.irc_message_filters.values())
         if hasattr(self, 'recurring'):
             gevent.spawn(self.run_recurring)
         self._register()
@@ -103,10 +92,8 @@ class TenyksService(object):
                         self.logger.debug("Got HELLO message; registering...")
                         self._register()
                         continue
-                    if self.direct_only and not data.get('direct', None):
-                        continue
                     if self.irc_message_filters and 'payload' in data:
-                        name, match = self.search_for_match(data['payload'])
+                        name, match = self.search_for_match(data)
                         ignore = (hasattr(self, 'pass_on_non_match')
                                   and self.pass_on_non_match)
                         if match or ignore:
@@ -116,12 +103,13 @@ class TenyksService(object):
             except ValueError:
                 self.logger.info('Invalid JSON. Ignoring message.')
 
-    def search_for_match(self, message):
-        for name, regexes in self.re_irc_message_filters.iteritems():
-            for regex in regexes:
-                match = regex(message)
-                if match:
-                    return name, match
+    def search_for_match(self, data):
+        for name, filter_chain in self.irc_message_filters.iteritems():
+            if filter_chain.direct_only and not data.get('direct', None):
+                continue
+            match = filter_chain.attempt_match(data['payload'])
+            if match:
+                return name, match
         return None, None
 
     def delegate_to_handle_method(self, data, match, name):
@@ -153,13 +141,38 @@ class TenyksService(object):
                 'meta': {
                     'name': self.name,
                     'version': self.version or 0.0,
-                    "UUID": self.settings.SERVICE_UUID,
-                    "description": self.settings.SERVICE_DESCRIPTION
+                    'UUID': self.settings.SERVICE_UUID,
+                    'description': self.settings.SERVICE_DESCRIPTION
                 }
             })
         r.publish(broadcast_channel, to_publish)
 
 
+class FilterChain(object):
+
+    def __init__(self, filters, direct_only=False):
+        self.filters = filters
+        self.direct_only = direct_only
+        self.compiled_filters = []
+
+    def _compile_filters(self):
+        if self.filters:
+            for f in self.filters:
+                if isinstance(f, str) or isinstance(f, unicode):
+                    self.compiled_filters.append(
+                        re.compile(f).match)
+                else:
+                    self.compiled_filters.append(f)  # already compiled filters
+
+    def attempt_match(self, message):
+        # tried to match message and returns the first one found or None
+        for f in self.compiled_filters:
+            match = f(message)
+            if match:
+                return match
+        return None
+
+
 def run_service(service_class):
     errors = collect_settings()
     service_instance = service_class(settings.SERVICE_NAME, settings)
